﻿// Copyright (c) Microsoft Corporation. All rights reserved.

namespace EditActionRecording
{
    using System;
    using System.IO;
    using System.Linq;
    using Microsoft.TeamFoundation.Client;
    using Microsoft.TeamFoundation.TestManagement.Client;
    using Microsoft.TeamFoundation.TestManagement.Common;
    using System.Collections.Generic;

    internal class Program
    {
        internal static void Main(string[] args)
        {
            // Get the server and project - may be by parsing the args.
            string serverName = "http://gautamg-tcm1:8080/tfs/defaultcollection";
            string projectName = "GautamG.unit.1";
            string testCaseQuery = "SELECT * FROM WorkItems WHERE [System.Id] IN ('2', '3')";

            ITestManagementTeamProject project = GetTestManagementProject(serverName, projectName);
            IEnumerable<ITestCase> testCases = project.TestCases.Query(testCaseQuery);
            UpdateActionRecordings(project, testCases);
        }

        private static ITestManagementTeamProject GetTestManagementProject(
            string serverName, string projectName)
        {
            var tfs = TfsTeamProjectCollectionFactory.GetTeamProjectCollection(new Uri(serverName));
            var testService = tfs.GetService<ITestManagementService>();
            return testService.GetTeamProject(projectName);
        }

        private static void UpdateActionRecordings(
            ITestManagementTeamProject project, IEnumerable<ITestCase> testCases)
        {
            foreach (var testCase in testCases)
            {
                ITestAttachment actionRecording = project.ActionRecordings.GetDefaultForTestCase(testCase.Id);
                if (actionRecording != null)
                {
                    UpdateActionRecording(project, testCase, actionRecording);
                }
            }
        }

        private static void UpdateActionRecording(
            ITestManagementTeamProject project, ITestCase testCase, ITestAttachment actionRecording)
        {
            // Download the attachment to a temp file.
            string localFileRecording = Path.GetTempFileName();
            try
            {
                actionRecording.DownloadToFile(localFileRecording);
                if (FixActionRecording(localFileRecording))
                {
                    UploadActionRecording(project, testCase, actionRecording, localFileRecording);
                }
            }
            finally
            {
                File.Delete(localFileRecording);
            }
        }

        // Return true if the recording has been changed.
        private static bool FixActionRecording(string localFileRecording)
        {
            // Do whatever edit you want to do here.
            // Check http://blogs.msdn.com/b/mathew_aniyan/archive/2011/03/31/howto-add-all-controls-on-a-page-to-the-ui-map.aspx

            // As an example, I am doing a dump replacement of all reference to
            // "2011" to "2012" here. HAPPY NEW YEAR 2012!!!
            string originalContents = File.ReadAllText(localFileRecording);
            bool has2011 = originalContents.IndexOf("2011", StringComparison.OrdinalIgnoreCase) >= 0;
            if (has2011)
            {
                string replacementContents = originalContents.Replace("2011", "2012");
                File.WriteAllText(localFileRecording, replacementContents);
            }

            return has2011;
        }

        private static void UploadActionRecording(
            ITestManagementTeamProject project, ITestCase testCase,
            ITestAttachment actionRecording, string localFileRecording)
        {
            ITestAttachment newActionRecording = project.ActionRecordings.Create(localFileRecording);
            newActionRecording.Comment = actionRecording.Comment;
            newActionRecording.Name = actionRecording.Name;

            UpdateTestCaseResult(project, testCase, newActionRecording);
            project.ActionRecordings.SetDefaultForTestCase(testCase.Id, newActionRecording);
        }

        private static void UpdateTestCaseResult(
            ITestManagementTeamProject project, ITestCase testCase, ITestAttachment actionRecording)
        {
            var result = project.TestResults.ByTestId(testCase.Id)[0];
            result.ActionRecording = actionRecording;
            result.Save();
        }
    }
}
